<?php
ini_set('display_errors', 1);
error_reporting(E_ALL);

$databasePath = __DIR__ . '/links.db';

if (!is_writable(__DIR__)) {
    die("Error: Directory " . __DIR__ . " is not writable. Run 'chown -R www:www /www/yourpath/todirection/linktr/' and 'chmod -R 775 /www/yourpath/todirection/linktr/'.");
}

try {
    $db = new SQLite3($databasePath);
    $db->exec("CREATE TABLE IF NOT EXISTS categories (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT, position INTEGER)");
    $db->exec("CREATE TABLE IF NOT EXISTS links (id INTEGER PRIMARY KEY AUTOINCREMENT, category_id INTEGER, name TEXT, url TEXT, position INTEGER)");
    $db->exec("CREATE TABLE IF NOT EXISTS settings (key TEXT PRIMARY KEY, value TEXT)");
    $db->exec("INSERT OR IGNORE INTO settings (key, value) VALUES ('logo', 'default-logo.png')");
    $db->exec("INSERT OR IGNORE INTO settings (key, value) VALUES ('title', 'Your Name Here')");
    $db->exec("INSERT OR IGNORE INTO settings (key, value) VALUES ('description', 'My important links')");
} catch (Exception $e) {
    die("Error opening database: " . $e->getMessage() . "<br>Path: " . $databasePath . "<br>Ensure php8.3-sqlite3 is installed ('sudo apt-get install php8.3-sqlite3').");
}

if (isset($_FILES['logo']) && $_SERVER['REQUEST_METHOD'] === 'POST') {
    $target_dir = __DIR__ . "/Uploads/";
    if (!is_dir($target_dir)) {
        if (!mkdir($target_dir, 0775, true)) {
            die("Error: Could not create upload directory.");
        }
        chmod($target_dir, 0775);
        chown($target_dir, 'www');
        chgrp($target_dir, 'www');
    }
    $target_file = $target_dir . basename($_FILES["logo"]["name"]);
    if (move_uploaded_file($_FILES["logo"]["tmp_name"], $target_file)) {
        $db->exec("INSERT OR REPLACE INTO settings (key, value) VALUES ('logo', 'Uploads/" . basename($target_file) . "')");
    } else {
        die("Error uploading logo.");
    }
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Handle add_category actions created by SaurusTV
    if (isset($_POST['add_category']) && is_array($_POST['add_category'])) {
        foreach ($_POST['add_category'] as $index => $data) {
            $name = trim($data['name']);
            if (!empty($name)) {
                $position = $db->querySingle("SELECT MAX(position) FROM categories") + 1 ?? 0;
                $stmt = $db->prepare("INSERT INTO categories (name, position) VALUES (:name, :position)");
                $stmt->bindValue(':name', $name, SQLITE3_TEXT);
                $stmt->bindValue(':position', $position, SQLITE3_INTEGER);
                $stmt->execute();
                // Store the new category ID for links referencing temp_id
                if (isset($data['temp_id'])) {
                    $new_category_id = $db->lastInsertRowID();
                    $_POST['add_category'][$index]['new_id'] = $new_category_id;
                }
            }
        }
    }

    // Handle add_link actions
    if (isset($_POST['add_link']) && is_array($_POST['add_link'])) {
        foreach ($_POST['add_link'] as $index => $data) {
            $category_id = (int)$data['category_id'];
            $name = trim($data['name']);
            $url = trim($data['url']);
            // If category_id is a temp_id, map it to the new category ID
            if (isset($data['category_temp_id'])) {
                foreach ($_POST['add_category'] as $cat) {
                    if (isset($cat['temp_id']) && $cat['temp_id'] === $data['category_temp_id'] && isset($cat['new_id'])) {
                        $category_id = (int)$cat['new_id'];
                        break;
                    }
                }
            }
            if (!empty($name) && !empty($url) && $category_id > 0) {
                $position = $db->querySingle("SELECT MAX(position) FROM links WHERE category_id = $category_id") + 1 ?? 0;
                $stmt = $db->prepare("INSERT INTO links (category_id, name, url, position) VALUES (:category_id, :name, :url, :position)");
                $stmt->bindValue(':category_id', $category_id, SQLITE3_INTEGER);
                $stmt->bindValue(':name', $name, SQLITE3_TEXT);
                $stmt->bindValue(':url', $url, SQLITE3_TEXT);
                $stmt->bindValue(':position', $position, SQLITE3_INTEGER);
                $stmt->execute();
            }
        }
    }

    // Handle delete_category actions
    if (isset($_POST['delete_category']) && is_array($_POST['delete_category'])) {
        foreach ($_POST['delete_category'] as $id) {
            $id = (int)$id;
            $db->exec("DELETE FROM categories WHERE id = $id");
            $db->exec("DELETE FROM links WHERE category_id = $id");
        }
    }

    // Handle delete_link actions
    if (isset($_POST['delete_link']) && is_array($_POST['delete_link'])) {
        foreach ($_POST['delete_link'] as $id) {
            $id = (int)$id;
            $db->exec("DELETE FROM links WHERE id = $id");
        }
    }

    // Handle edit_category actions
    if (isset($_POST['edit_category']) && is_array($_POST['edit_category'])) {
        foreach ($_POST['edit_category'] as $id => $data) {
            $name = trim($data['name']);
            if (!empty($name)) {
                $stmt = $db->prepare("UPDATE categories SET name = :name WHERE id = :id");
                $stmt->bindValue(':name', $name, SQLITE3_TEXT);
                $stmt->bindValue(':id', (int)$id, SQLITE3_INTEGER);
                $stmt->execute();
            }
        }
    }

    // Handle edit_link actions
    if (isset($_POST['edit_link']) && is_array($_POST['edit_link'])) {
        foreach ($_POST['edit_link'] as $id => $data) {
            $name = trim($data['name']);
            $url = trim($data['url']);
            if (!empty($name) && !empty($url)) {
                $stmt = $db->prepare("UPDATE links SET name = :name, url = :url WHERE id = :id");
                $stmt->bindValue(':name', $name, SQLITE3_TEXT);
                $stmt->bindValue(':url', $url, SQLITE3_TEXT);
                $stmt->bindValue(':id', (int)$id, SQLITE3_INTEGER);
                $stmt->execute();
            }
        }
    }

    // Handle edit_title
    if (isset($_POST['edit_title'])) {
        $title = trim($_POST['edit_title']);
        if (!empty($title)) {
            $stmt = $db->prepare("INSERT OR REPLACE INTO settings (key, value) VALUES ('title', :title)");
            $stmt->bindValue(':title', $title, SQLITE3_TEXT);
            $stmt->execute();
        }
    }

    // Handle edit_description
    if (isset($_POST['edit_description'])) {
        $description = trim($_POST['edit_description']);
        if (!empty($description)) {
            $stmt = $db->prepare("INSERT OR REPLACE INTO settings (key, value) VALUES ('description', :description)");
            $stmt->bindValue(':description', $description, SQLITE3_TEXT);
            $stmt->execute();
        }
    }

    // Handle move_category actions @SaurusKing
    if (isset($_POST['move_category']) && is_array($_POST['move_category'])) {
        foreach ($_POST['move_category'] as $id => $data) {
            $direction = $data['direction'];
            $id = (int)$id;
            $current_pos = $db->querySingle("SELECT position FROM categories WHERE id = $id");
            $new_pos = $direction === 'up' ? $current_pos - 1 : $current_pos + 1;
            if ($new_pos >= 0) {
                $db->exec("UPDATE categories SET position = $current_pos WHERE position = $new_pos");
                $db->exec("UPDATE categories SET position = $new_pos WHERE id = $id");
            }
        }
    }

    // Handle update_positions
    if (isset($_POST['update_positions'])) {
        $positions = json_decode($_POST['update_positions'], true);
        foreach ($positions as $link_id => $data) {
            $category_id = (int)$data['category_id'];
            $position = (int)$data['position'];
            $link_id = (int)$link_id;
            if ($category_id > 0 && $link_id > 0) {
                $stmt = $db->prepare("UPDATE links SET category_id = :category_id, position = :position WHERE id = :id");
                $stmt->bindValue(':category_id', $category_id, SQLITE3_INTEGER);
                $stmt->bindValue(':position', $position, SQLITE3_INTEGER);
                $stmt->bindValue(':id', $link_id, SQLITE3_INTEGER);
                $stmt->execute();
            }
        }
    }
}

$logo = $db->querySingle("SELECT value FROM settings WHERE key = 'logo'");
$title = $db->querySingle("SELECT value FROM settings WHERE key = 'title'");
$description = $db->querySingle("SELECT value FROM settings WHERE key = 'description'");
$categories = $db->query("SELECT * FROM categories ORDER BY position");
$links = [];
$result = $db->query("SELECT * FROM links ORDER BY category_id, position");
while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
    $links[$row['category_id']][] = $row;
}
?>

<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title><?php echo htmlspecialchars($title); ?> Links</title>
    <link rel="apple-touch-icon" sizes="180x180" href="<?php echo htmlspecialchars($logo); ?>">
    <link rel="icon" type="image/png" sizes="32x32" href="<?php echo htmlspecialchars($logo); ?>">
    <link rel="icon" type="image/png" sizes="16x16" href="<?php echo htmlspecialchars($logo); ?>">
    <meta name="apple-mobile-web-app-title" content="<?php echo htmlspecialchars($title); ?>">
    <meta name="apple-mobile-web-app-capable" content="yes">
    <meta name="apple-mobile-web-app-status-bar-style" content="black">
    <style>
        * { margin: 1; padding: 0; box-sizing: border-box; font-family: 'Arial', sans-serif; }
        body { background: linear-gradient(135deg, #2c3e50 0%, #3a506b 100%); min-height: 100vh; padding: 20px; padding-top: 20px; display: flex; justify-content: center; align-items: center; }
        .container { max-width: 680px; width: 100%; padding: 20px; background: rgba(255, 255, 255, 0.1); backdrop-filter: blur(10px); border-radius: 20px; box-shadow: 0 8px 32px rgba(0, 0, 0, 0.3); }
        .profile { text-align: center; margin-bottom: 30px; }
        .profile-img { width: 300px; height: 200px; object-fit: contain; border-radius: 15px; margin-bottom: 15px; box-shadow: 0 6px 20px rgba(0, 0, 0, 0.4); transition: transform 0.3s ease; Blaise }
        .profile-img:hover { transform: scale(1.05); }
        body.edit-mode .profile-img { cursor: pointer; }
        .profile h1 { color: #e5e5e5; font-size: 24px; margin-bottom: 10px; text-shadow: 0 2px 4px rgba(0, 0, 0, 0.3); }
        body.edit-mode .profile h1 { cursor: pointer; }
        .profile p { color: #e5e5e5; opacity: 0.8; font-size: 16px; margin-bottom: 20px; text-shadow: 0 2px 4px rgba(0, 0, 0, 0.3); }
        body.edit-mode .profile p { cursor: pointer; }
        .links { display: flex; flex-direction: column; gap: 16px; padding: 15px; }
        .link { background: linear-gradient(145deg, #414f5d, #2d3741); padding: 15px 20px; border-radius: 12px; text-decoration: none; color: #e5e5e5; font-weight: 500; text-align: center; transition: all 0.3s ease; display: flex; align-items: center; justify-content: center; border: 1px solid rgba(255, 255, 255, 0.1); box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2); }
        .link:hover { background: linear-gradient(145deg, #516273, #3a4754); transform: translateY(-3px); box-shadow: 0 6px 20px rgba(0, 0, 0, 0.3); }
        .categories { display: flex; flex-direction: column; gap: 20px; margin-bottom: 20px; }
        details { background: rgba(255, 255, 255, 0.05); border-radius: 12px; overflow: hidden; border: 1px solid rgba(255, 255, 255, 0.1); }
        summary { padding: 15px 60px 15px 20px; font-weight: bold; color: #e5e5e5; cursor: pointer; background: linear-gradient(145deg, #414f5d, #2d3741); position: relative; outline: none; transition: all 0.3s ease; list-style: none; }
        summary::-webkit-details-marker { display: none; }
        summary::after { content: "▼"; position: absolute; right: 15px; top: 50%; transform: translateY(-50%); transition: transform 0.3s ease; }
        details[open] summary::after { transform: translateY(-50%) rotate(180deg); }
        summary:hover { background: linear-gradient(145deg, #516273, #3a4754); }
        .edit-controls { display: flex; justify-content: flex-end; margin-bottom: 20px; gap: 10px; }
        .edit-toggle, .save-button { background: linear-gradient(145deg, #414f5d, #2d3741); border: 1px solid rgba(255, 255, 255, 0.1); color: #e5e5e5; border-radius: 50%; width: 40px; height: 40px; display: flex; align-items: center; justify-content: center; cursor: pointer; transition: all 0.3s ease; font-size: 20px; box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2); }
        .edit-toggle:hover, .save-button:hover { background: linear-gradient(145deg, #516273, #3a4754); transform: translateY(-2px); }
        .save-button { display: none; position: relative; }
        .save-button.saving::after { content: "Saving..."; position: absolute; right: 50px; color: #e5e5e5; font-size: 14px; }
        body.edit-mode .save-button { display: flex; }
        .add-button, .move-button { background: linear-gradient(145deg, #414f5d, #2d3741); border: none; color: #e5e5e5; border-radius: 12px; padding: 10px 20px; cursor: pointer; transition: all 0.3s ease; box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2); }
        .add-button:hover, .move-button:hover { background: linear-gradient(145deg, #516273, #3a4754); transform: translateY(-2px); }
        .add-button { width: 100%; margin: 10px 0; }
        .edit-mode-controls { display: none; }
        body.edit-mode .edit-mode-controls { display: block; }
        body.edit-mode .link { cursor: move; position: relative; padding-left: 40px; padding-right: 40px; }
        body.edit-mode .link::before { content: "≡"; position: absolute; left: 15px; color: rgba(255, 255, 255, 0.7); }
        body.edit-mode .link::after { content: "✕"; position: absolute; right: 15px; color: rgba(255, 255, 255, 0.7); cursor: pointer; }
        body.edit-mode summary::after { content: "✕"; right: 15px; }
        body.edit-mode summary::before { content: "⚙️"; position: absolute; right: 40px; color: rgba(255, 255, 255, 0.7); cursor: pointer; }
        .link.dragging { opacity: 0.5; }
        .link.drag-over { border-top: 2px solid rgba(255, 255, 255, 0.8); }
        .popup { display: none; position: fixed; top: 50%; left: 50%; transform: translate(-50%, -50%); background: rgba(255, 255, 255, 0.1); backdrop-filter: blur(10px); padding: 20px; border-radius: 12px; box-shadow: 0 8px 32px rgba(0, 0, 0, 0.3); z-index: 1000; width: 300px; }
        .popup.active { display: block; }
        .popup form { display: flex; flex-direction: column; gap: 10px; }
        .popup input { padding: 8px; border-radius: 8px; border: 1px solid rgba(255, 255, 255, 0.2); background: rgba(255, 255, 255, 0.05); color: #e5e5e5; }
        .popup button { background: linear-gradient(145deg, #516273, #3a4754); border: none; color: #e5e5e5; border-radius: 12px; padding: 10px; cursor: pointer; transition: all 0.3s ease; }
        .popup button:hover { background: linear-gradient(145deg, #617383, #46515e); }
        .overlay { display: none; position: fixed; top: 0; left: 0; width: 100%; height: 100%; background: rgba(0, 0, 0, 0.5); z-index: 999; }
        .overlay.active { display: block; }
        #logoInput { display: none; }
    </style>
</head>
<body>
    <div class="container">
        <div class="profile">
            <img src="<?php echo htmlspecialchars($logo); ?>" alt="Logo" class="profile-img" id="profileImg">
            <input type="file" id="logoInput" accept="image/*">
            <h1 id="profileTitle"><?php echo htmlspecialchars($title); ?></h1>
            <p id="profileDescription"><?php echo htmlspecialchars($description); ?></p>
        </div>
        <div class="edit-controls">
            <button id="editModeToggle" class="edit-toggle">⚙️</button>
            <button id="saveChanges" class="save-button">💾</button>
        </div>
        <div class="categories">
            <?php while ($category = $categories->fetchArray(SQLITE3_ASSOC)): ?>
                <details>
                    <summary data-id="<?php echo $category['id']; ?>"><?php echo htmlspecialchars($category['name']); ?></summary>
                    <div class="links">
                        <?php if (isset($links[$category['id']])): foreach ($links[$category['id']] as $link): ?>
                            <a href="<?php echo htmlspecialchars($link['url']); ?>" class="link" data-id="<?php echo $link['id']; ?>" draggable="false"><?php echo htmlspecialchars($link['name']); ?></a>
                        <?php endforeach; endif; ?>
                    </div>
                    <div class="edit-mode-controls">
                        <button class="add-button add-link-button" data-category-id="<?php echo $category['id']; ?>">+ New Link</button>
                        <button class="move-button" data-action="move_category" data-id="<?php echo $category['id']; ?>" data-direction="up">↑</button>
                        <button class="move-button" data-action="move_category" data-id="<?php echo $category['id']; ?>" data-direction="down">↓</button>
                    </div>
                </details>
            <?php endwhile; ?>
            <div class="edit-mode-controls">
                <button id="addCategoryButton" class="add-button">+ New Category</button>
            </div>
        </div>
    </div>

    <div class="overlay" id="popupOverlay"></div>
    <div class="popup" id="editPopup"></div>

    <script>
        document.addEventListener('DOMContentLoaded', function() {
            const editToggle = document.getElementById('editModeToggle');
            const saveButton = document.getElementById('saveChanges');
            const popupOverlay = document.getElementById('popupOverlay');
            const editPopup = document.getElementById('editPopup');
            const categoriesContainer = document.querySelector('.categories');
            const profileImg = document.getElementById('profileImg');
            const logoInput = document.getElementById('logoInput');
            const profileTitle = document.getElementById('profileTitle');
            const profileDescription = document.getElementById('profileDescription');
            let isEditMode = false;
            let isSaving = false;
            let pendingChanges = {
                addCategories: [],
                addLinks: [],
                deleteCategories: [],
                deleteLinks: [],
                editCategories: {},
                editLinks: {},
                moveCategories: {},
                updatePositions: {},
                editTitle: null,
                editDescription: null
            };
            let pendingLogo = null;

            editToggle.addEventListener('click', function() {
                isEditMode = !isEditMode;
                document.body.classList.toggle('edit-mode', isEditMode);
                if (isEditMode) enableEditMode();
                else disableEditMode();
            });

            saveButton.addEventListener('click', function() {
                if (!isSaving) {
                    saveChanges();
                }
            });

            popupOverlay.addEventListener('click', closePopup);

            function enableEditMode() {
                document.querySelectorAll('.link').forEach(link => {
                    link.addEventListener('click', handleLinkEdit);
                });
                document.querySelectorAll('summary').forEach(summary => {
                    summary.addEventListener('click', handleSummaryEdit);
                });
                document.querySelectorAll('.add-link-button').forEach(button => {
                    button.addEventListener('click', handleAddLinkPopup);
                });
                document.getElementById('addCategoryButton').addEventListener('click', handleAddCategoryPopup);
                document.querySelectorAll('.move-button').forEach(button => {
                    button.addEventListener('click', handleMoveCategory);
                });
                profileImg.addEventListener('click', handleLogoChange);
                profileTitle.addEventListener('click', handleTitleEdit);
                profileDescription.addEventListener('click', handleDescriptionEdit);
                enableDragAndDrop();
            }

            function disableEditMode() {
                document.querySelectorAll('.link').forEach(link => {
                    link.removeEventListener('click', handleLinkEdit);
                });
                document.querySelectorAll('summary').forEach(summary => {
                    summary.removeEventListener('click', handleSummaryEdit);
                });
                document.querySelectorAll('.add-link-button').forEach(button => {
                    button.removeEventListener('click', handleAddLinkPopup);
                });
                document.getElementById('addCategoryButton').removeEventListener('click', handleAddCategoryPopup);
                document.querySelectorAll('.move-button').forEach(button => {
                    button.removeEventListener('click', handleMoveCategory);
                });
                profileImg.removeEventListener('click', handleLogoChange);
                profileTitle.removeEventListener('click', handleTitleEdit);
                profileDescription.removeEventListener('click', handleDescriptionEdit);
                disableDragAndDrop();
                closePopup();
                // Reset pending changes
                pendingChanges = {
                    addCategories: [],
                    addLinks: [],
                    deleteCategories: [],
                    deleteLinks: [],
                    editCategories: {},
                    editLinks: {},
                    moveCategories: {},
                    updatePositions: {},
                    editTitle: null,
                    editDescription: null
                };
                pendingLogo = null;
            }

            function handleLogoChange() {
                logoInput.click();
            }

            logoInput.addEventListener('change', function(e) {
                const file = e.target.files[0];
                if (file) {
                    const reader = new FileReader();
                    reader.onload = function(e) {
                        profileImg.src = e.target.result;
                        pendingLogo = file;
                    };
                    reader.readAsDataURL(file);
                }
            });

            function handleTitleEdit(e) {
                e.preventDefault();
                editPopup.innerHTML = `
                    <form id="editTitleForm">
                        <input type="text" name="title" value="${profileTitle.textContent}" required placeholder="Enter title">
                        <button type="submit">Save</button>
                    </form>
                `;
                editPopup.classList.add('active');
                popupOverlay.classList.add('active');
                const form = document.getElementById('editTitleForm');
                form.addEventListener('submit', function(e) {
                    e.preventDefault();
                    const title = form.querySelector('[name="title"]').value.trim();
                    if (title) {
                        pendingChanges.editTitle = title;
                        profileTitle.textContent = title;
                        closePopup();
                    }
                });
            }

            function handleDescriptionEdit(e) {
                e.preventDefault();
                editPopup.innerHTML = `
                    <form id="editDescriptionForm">
                        <input type="text" name="description" value="${profileDescription.textContent}" required placeholder="Enter description">
                        <button type="submit">Save</button>
                    </form>
                `;
                editPopup.classList.add('active');
                popupOverlay.classList.add('active');
                const form = document.getElementById('editDescriptionForm');
                form.addEventListener('submit', function(e) {
                    e.preventDefault();
                    const description = form.querySelector('[name="description"]').value.trim();
                    if (description) {
                        pendingChanges.editDescription = description;
                        profileDescription.textContent = description;
                        closePopup();
                    }
                });
            }

            function handleLinkEdit(e) {
                e.preventDefault();
                const link = e.target.closest('.link');
                const rect = link.getBoundingClientRect();
                const clickX = e.clientX - rect.left;
                const linkWidth = rect.width;

                if (clickX > linkWidth - 30) {
                    if (confirm('Delete link?')) {
                        const linkId = link.dataset.id;
                        pendingChanges.deleteLinks.push(linkId);
                        link.remove();
                    }
                } else {
                    editPopup.innerHTML = `
                        <form id="editLinkForm">
                            <input type="text" name="link_name" value="${link.textContent}" required placeholder="Link name">
                            <input type="url" name="link_url" value="${link.href}" required placeholder="Link URL (with https://)">
                            <button type="submit">Save</button>
                        </form>
                    `;
                    editPopup.classList.add('active');
                    popupOverlay.classList.add('active');
                    const form = document.getElementById('editLinkForm');
                    form.addEventListener('submit', function(e) {
                        e.preventDefault();
                        const name = form.querySelector('[name="link_name"]').value.trim();
                        const url = form.querySelector('[name="link_url"]').value.trim();
                        if (name && url) {
                            const linkId = link.dataset.id;
                            pendingChanges.editLinks[linkId] = { name, url };
                            link.textContent = name;
                            link.href = url;
                            closePopup();
                        }
                    });
                }
            }

            function handleSummaryEdit(e) {
                const summary = e.target.closest('summary');
                const rect = summary.getBoundingClientRect();
                const clickX = e.clientX - rect.left;
                const summaryWidth = rect.width;

                if (clickX > summaryWidth - 65 && clickX < summaryWidth - 25) {
                    e.preventDefault();
                    editPopup.innerHTML = `
                        <form id="editCategoryForm">
                            <input type="text" name="category_name" value="${summary.textContent.trim()}" required placeholder="Category name">
                            <button type="submit">Save</button>
                        </form>
                    `;
                    editPopup.classList.add('active');
                    popupOverlay.classList.add('active');
                    const form = document.getElementById('editCategoryForm');
                    form.addEventListener('submit', function(e) {
                        e.preventDefault();
                        const name = form.querySelector('[name="category_name"]').value.trim();
                        if (name) {
                            const categoryId = summary.dataset.id;
                            pendingChanges.editCategories[categoryId] = { name };
                            summary.textContent = name;
                            closePopup();
                        }
                    });
                } else if (clickX > summaryWidth - 25) {
                    e.preventDefault();
                    if (confirm('Delete category?')) {
                        const categoryId = summary.dataset.id;
                        pendingChanges.deleteCategories.push(categoryId);
                        summary.parentNode.remove();
                    }
                }
            }

            function handleAddLinkPopup(e) {
                e.preventDefault();
                const categoryId = e.target.dataset.categoryId;
                const categoryLinks = document.querySelector(`summary[data-id="${categoryId}"]`).parentNode.querySelector('.links');
                editPopup.innerHTML = `
                    <form id="addLinkForm">
                        <input type="text" name="link_name" placeholder="Link name" required>
                        <input type="url" name="link_url" placeholder="Link URL (with https://)" required>
                        <button type="submit">Add</button>
                    </form>
                `;
                editPopup.classList.add('active');
                popupOverlay.classList.add('active');
                const form = document.getElementById('addLinkForm');
                form.addEventListener('submit', function(e) {
                    e.preventDefault();
                    const name = form.querySelector('[name="link_name"]').value.trim();
                    const url = form.querySelector('[name="link_url"]').value.trim();
                    if (name && url) {
                        const tempId = 'temp-link-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);
                        const newLink = document.createElement('a');
                        newLink.href = url;
                        newLink.className = 'link';
                        newLink.textContent = name;
                        newLink.dataset.id = tempId;
                        newLink.draggable = true;
                        categoryLinks.appendChild(newLink);
                        pendingChanges.addLinks.push({ temp_id: tempId, category_id: categoryId, name, url });
                        closePopup();
                        if (isEditMode) newLink.addEventListener('click', handleLinkEdit);
                    }
                });
            }

            function handleAddCategoryPopup(e) {
                e.preventDefault();
                editPopup.innerHTML = `
                    <form id="addCategoryForm">
                        <input type="text" name="category_name" placeholder="New category" required>
                        <button type="submit">Add</button>
                    </form>
                `;
                editPopup.classList.add('active');
                popupOverlay.classList.add('active');
                const form = document.getElementById('addCategoryForm');
                form.addEventListener('submit', function(e) {
                    e.preventDefault();
                    const name = form.querySelector('[name="category_name"]').value.trim();
                    if (name) {
                        const tempId = 'temp-cat-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);
                        const newDetails = document.createElement('details');
                        newDetails.innerHTML = `
                            <summary data-id="${tempId}">${name}</summary>
                            <div class="links"></div>
                            <div class="edit-mode-controls">
                                <button class="add-button add-link-button" data-category-id="${tempId}">+ New Link</button>
                                <button class="move-button" data-action="move_category" data-id="${tempId}" data-direction="up">↑</button>
                                <button class="move-button" data-action="move_category" data-id="${tempId}" data-direction="down">↓</button>
                            </div>
                        `;
                        categoriesContainer.insertBefore(newDetails, categoriesContainer.lastElementChild);
                        pendingChanges.addCategories.push({ temp_id: tempId, name });
                        closePopup();
                        if (isEditMode) {
                            newDetails.querySelector('summary').addEventListener('click', handleSummaryEdit);
                            newDetails.querySelector('.add-link-button').addEventListener('click', handleAddLinkPopup);
                            newDetails.querySelectorAll('.move-button').forEach(button => button.addEventListener('click', handleMoveCategory));
                        }
                    }
                });
            }

            function handleMoveCategory(e) {
                const button = e.target;
                const categoryId = button.dataset.id;
                const direction = button.dataset.direction;
                pendingChanges.moveCategories[categoryId] = { direction };
                const details = button.closest('details');
                const parent = details.parentNode;
                if (direction === 'up' && details.previousElementSibling && !details.previousElementSibling.classList.contains('edit-mode-controls')) {
                    parent.insertBefore(details, details.previousElementSibling);
                } else if (direction === 'down' && details.nextElementSibling && !details.nextElementSibling.classList.contains('edit-mode-controls')) {
                    parent.insertBefore(details.nextElementSibling, details);
                }
            }

            let draggedItem = null;
            function enableDragAndDrop() {
                document.querySelectorAll('.link').forEach(link => {
                    link.setAttribute('draggable', true);
                    link.addEventListener('dragstart', handleDragStart);
                    link.addEventListener('dragover', handleDragOver);
                    link.addEventListener('dragenter', handleDragEnter);
                    link.addEventListener('dragleave', handleDragLeave);
                    link.addEventListener('drop', handleDrop);
                    link.addEventListener('dragend', handleDragEnd);
                });
            }
            function disableDragAndDrop() {
                document.querySelectorAll('.link').forEach(link => {
                    link.setAttribute('draggable', false);
                    link.removeEventListener('dragstart', handleDragStart);
                    link.removeEventListener('dragover', handleDragOver);
                    link.removeEventListener('dragenter', handleDragEnter);
                    link.removeEventListener('dragleave', handleDragLeave);
                    link.removeEventListener('drop', handleDrop);
                    link.removeEventListener('dragend', handleDragEnd);
                });
            }
            function handleDragStart(e) {
                draggedItem = this;
                this.classList.add('dragging');
                e.dataTransfer.setData('text/plain', '');
            }
            function handleDragOver(e) {
                e.preventDefault();
            }
            function handleDragEnter(e) {
                e.preventDefault();
                this.classList.add('drag-over');
            }
            function handleDragLeave() {
                this.classList.remove('drag-over');
            }
            function handleDrop(e) {
                e.preventDefault();
                this.classList.remove('drag-over');
                if (draggedItem !== this) {
                    const toContainer = this.parentNode;
                    const categoryId = toContainer.parentNode.querySelector('summary').dataset.id;
                    const linkId = draggedItem.dataset.id;
                    toContainer.insertBefore(draggedItem, this);
                    updatePendingPositions();
                }
            }
            function handleDragEnd() {
                this.classList.remove('dragging');
                draggedItem = null;
            }

            function updatePendingPositions() {
                pendingChanges.updatePositions = {};
                document.querySelectorAll('.link').forEach((link, index) => {
                    const linkId = link.dataset.id;
                    const categoryId = link.closest('details').querySelector('summary').dataset.id;
                    pendingChanges.updatePositions[linkId] = { category_id: categoryId, position: index };
                });
            }

            function saveChanges() {
                // Check if there are any changes to save
                const hasChanges =
                    pendingChanges.addCategories.length > 0 ||
                    pendingChanges.addLinks.length > 0 ||
                    pendingChanges.deleteCategories.length > 0 ||
                    pendingChanges.deleteLinks.length > 0 ||
                    Object.keys(pendingChanges.editCategories).length > 0 ||
                    Object.keys(pendingChanges.editLinks).length > 0 ||
                    Object.keys(pendingChanges.moveCategories).length > 0 ||
                    Object.keys(pendingChanges.updatePositions).length > 0 ||
                    pendingChanges.editTitle ||
                    pendingChanges.editDescription ||
                    pendingLogo;

                if (!hasChanges) {
                    alert('No changes to save.');
                    return;
                }

                isSaving = true;
                saveButton.classList.add('saving');
                saveButton.disabled = true;

                const formData = new FormData();
                if (pendingLogo) {
                    formData.append('logo', pendingLogo);
                }

                // Add categories
                pendingChanges.addCategories.forEach((cat, index) => {
                    formData.append(`add_category[${index}][name]`, cat.name);
                    formData.append(`add_category[${index}][temp_id]`, cat.temp_id);
                });

                // Add links
                pendingChanges.addLinks.forEach((link, index) => {
                    formData.append(`add_link[${index}][category_id]`, link.category_id);
                    formData.append(`add_link[${index}][name]`, link.name);
                    formData.append(`add_link[${index}][url]`, link.url);
                    formData.append(`add_link[${index}][category_temp_id]`, link.category_id.startsWith('temp-cat-') ? link.category_id : '');
                });

                // Delete categories
                pendingChanges.deleteCategories.forEach(id => {
                    formData.append('delete_category[]', id);
                });

                // Delete links
                pendingChanges.deleteLinks.forEach(id => {
                    formData.append('delete_link[]', id);
                });

                // Edit categories
                Object.keys(pendingChanges.editCategories).forEach(id => {
                    formData.append(`edit_category[${id}][name]`, pendingChanges.editCategories[id].name);
                });

                // Edit links
                Object.keys(pendingChanges.editLinks).forEach(id => {
                    formData.append(`edit_link[${id}][name]`, pendingChanges.editLinks[id].name);
                    formData.append(`edit_link[${id}][url]`, pendingChanges.editLinks[id].url);
                });

                // Move categories
                Object.keys(pendingChanges.moveCategories).forEach(id => {
                    formData.append(`move_category[${id}][direction]`, pendingChanges.moveCategories[id].direction);
                });

                // Update positions
                if (Object.keys(pendingChanges.updatePositions).length > 0) {
                    formData.append('update_positions', JSON.stringify(pendingChanges.updatePositions));
                }

                // Edit title
                if (pendingChanges.editTitle) {
                    formData.append('edit_title', pendingChanges.editTitle);
                }

                // Edit description
                if (pendingChanges.editDescription) {
                    formData.append('edit_description', pendingChanges.editDescription);
                }

                fetch('', {
                    method: 'POST',
                    body: formData
                })
                .then(response => {
                    if (!response.ok) {
                        throw new Error('Network response was not ok');
                    }
                    // Reset pending changes and reload @SaurusKing
                    pendingChanges = {
                        addCategories: [],
                        addLinks: [],
                        deleteCategories: [],
                        deleteLinks: [],
                        editCategories: {},
                        editLinks: {},
                        moveCategories: {},
                        updatePositions: {},
                        editTitle: null,
                        editDescription: null
                    };
                    pendingLogo = null;
                    location.reload();
                })
                .catch(error => {
                    console.error('Error saving changes:', error);
                    alert('Failed to save changes. Please try again.');
                    isSaving = false;
                    saveButton.classList.remove('saving');
                    saveButton.disabled = false;
                });
            }

            function closePopup() {
                editPopup.classList.remove('active');
                popupOverlay.classList.remove('active');
            }
        });
    </script>
</body>
</html>